//----------------------------------------------------------------------
//
//  Modem Control Demo for DNP/5280
//
//----------------------------------------------------------------------
//  FILE    : modemctrl.c
//  AUTHOR  : Marco Hasewinkel (mha@ssv)
//  EMAIL   : <info@dilnetpc.com> <m.hasewinkel@web.de>
//  DATE    : 20.10.2004
//  VERSION : 1.00
//  LICENSE : GPL
//----------------------------------------------------------------------
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//----------------------------------------------------------------------

#include <unistd.h>
#include <stdio.h>

#include "ssvhwa.h"

#define MCFBAR  0x40000000

#define PORTTC   (MCFBAR + 0x0010000F)	/* 8 bit */
#define PORTTD   (MCFBAR + 0x00100010)	/* 8 bit */

#define DDRTC    (MCFBAR + 0x00100023)	/* 8 bit */
#define DDRTD    (MCFBAR + 0x00100024)	/* 8 bit */

#define PORTTCP  (MCFBAR + 0x00100037)	/* 8 bit */
#define PORTTDP  (MCFBAR + 0x00100038)	/* 8 bit */

#define PTCPAR   (MCFBAR + 0x0010005A)	/* 8 bit */
#define PTDPAR   (MCFBAR + 0x0010005B)	/* 8 bit */

void rts_pin(int state)
{
    if (state)
	ssvhwa_write8(PORTTD, ssvhwa_read8(PORTTD) | 0x04);
    else
	ssvhwa_write8(PORTTD, ssvhwa_read8(PORTTD) & ~0x04);
}

int cts_pin(void)
{
    return (ssvhwa_read8(PORTTDP) & 0x01);
}

void dtr_pin(int state)
{
    if (state)
	ssvhwa_write8(PORTTC, ssvhwa_read8(PORTTC) | 0x04);
    else
	ssvhwa_write8(PORTTC, ssvhwa_read8(PORTTC) & ~0x04);
}

int dsr_pin(void)
{
    return ((ssvhwa_read8(PORTTCP) >> 1) & 0x01);
}

int dcd_pin(void)
{
    return (ssvhwa_read8(PORTTCP) & 0x01);
}

int ri_pin(void)
{
    return ((ssvhwa_read8(PORTTCP) >> 3) & 0x01);
}

int main(void)
{
    /* get user identity */
    if (geteuid() != 0) {
	fprintf(stderr, "No root access rights !\n");
	exit(1);
    }

    if (ssvhwa_open() < 0) {		// open ssvhwa driver
	perror("ssvhwa open");
	exit(-1);
    }

    ssvhwa_write8(PTDPAR, 0x00);	//Pin TD0-TD3 to GPIO
    ssvhwa_write8(PTCPAR, 0x00);	//Pin TC0-TC3 to GPIO

    ssvhwa_write8(DDRTD, 0x04);		//RTS=output, CTS=input
    ssvhwa_write8(DDRTC, 0x04);		//DTR=output, DSR,DCD,RI=input

    for (;;) {

	rts_pin(0);			// set RTS pin low
	dtr_pin(0);			// set DTR pin low

	// show pin states
	printf("\rRTS:%d, CTS:%d, DTR:%d, DSR:%d, DCD:%d, RI:%d",
		0, cts_pin(), 0, dsr_pin(), dcd_pin(), ri_pin());

	fflush(stdout);
	sleep(1);


	rts_pin(1);			// set RTS pin high
	dtr_pin(1);			// set DTR pin high

	// show pin states
	printf("\rRTS:%d, CTS:%d, DTR:%d, DSR:%d, DCD:%d, RI:%d",
		1, cts_pin(), 1, dsr_pin(), dcd_pin(), ri_pin());

	fflush(stdout);
	sleep(1);

    }

    ssvhwa_close();

    return 0;
}
